# This file is a portion of the Red Hat Network Panel Applet
#
# Copyright (C) 1999-2002 Red Hat, Inc. All Rights Reserved.
# Distributed under GPL version 2.
#
# Author: Chip Turner
#
# $Id: rhn_applet_rpm.py,v 1.8 2002/09/03 07:23:11 gafton Exp $

# we need to get to the rpm404 compatible bindings
# be nice to older pythons (avoid import <foo> as <bar>)
try:
    import rpm404
except ImportError:
    import rpm
else:
    rpm = rpm404
if not hasattr(rpm, "opendb"):
    raise ImportError("Your %s module does not support "\
                      "RPM 4.0 API" % rpm.__name__)
import string
import os
import time

import gettext
import rhn_utils

GETTEXT_DOMAIN = "rhn-applet"
gettext.bindtextdomain (GETTEXT_DOMAIN, "/usr/share/locale")
gettext.textdomain (GETTEXT_DOMAIN)
try:
    gettext.install(GETTEXT_DOMAIN, "/usr/share/locale", 1)
except IOError:
    import __builtin__
    __builtin__.__dict__['_'] = unicode
except AttributeError: # older gettext modules don't have an install() method
    from gettext import _

class rhnAppletRPM:
    def __init__(self, fatal = 0):
        self.latest_installed_package_cache = None

        self.fatal = fatal
        self.last_rpmdb_mtime = 0
        self.change_number = 0

    def rpmdb_mtime(self):
        return os.stat('/var/lib/rpm/Packages')[8]

    def find_by_iter(self, iter_name, *values):
        rhn_utils.log_debug("Beginning RPM iter (%s, %s)" % (
            iter_name, values,))
        try:
            rpmdb = rpm.opendb()
        except rpm.error, e:
            if self.fatal:
                raise _("cannot open RPM database, perhaps it is not readable as this user, or already open?")
            else:
                rhn_utils.log_debug("Unable to open rpm database. Perhaps another application has "+\
                          "it open?")
                # placeholder value that gets overwritten when the db can be read again
                return []

        method = getattr(rpmdb, iter_name)
        ret = []
        iterator = apply(method, values)

        if type(iterator) == type([]):
            for index in iterator:
                i_tmp = {}
                for field in ("name", "version", "release", "epoch"):
                    i_tmp[field] = rpmdb[index][field]

                ret.append(i_tmp)
                
        else:
            h = iterator.next()
            while(h):
                i_tmp = {}
                for field in ("name", "version", "release", "epoch"):
                    i_tmp[field] = h[field]

                ret.append(i_tmp)
                h = iterator.next()

        rhn_utils.log_debug("Finished RPM query (%s, %s)" % (iter_name, values,))
        return ret

    def find_latest_by_name(self, name):
        h = self.latest_installed_packages()
        for i in h.keys():
            if h[i]["name"] == name:
                return h[i]
        return

    def find_provides(self, name):
        return self.find_by_iter("findbyprovides", name)
            
    def find_provides_by_file(self, filename):
        return self.find_by_iter("findbyfile", filename)
            
    def refresh(self, force=0):
        rhn_utils.log_debug("rpm db mtimes: %s, %s" % (self.last_rpmdb_mtime, self.rpmdb_mtime()))
        
        if not force and self.last_rpmdb_mtime == self.rpmdb_mtime() and self.latest_installed_package_cache:
            return
        
        rhn_utils.log_debug("Beginning RPM query for installed packages")
        try:
            rpmdb = rpm.opendb()
        except rpm.error, e:
            if self.fatal:
                raise _("cannot open RPM database, perhaps it is not readable as this user, or already open?")
            else:
                rhn_utils.log_debug("RPM error: %s" % str(e))

                # do we have an old cached version of data?  if so,
                # let's use it, since it is better than nothing
                
                if self.latest_installed_package_cache:
                    return self.latest_installed_package_cache
                else:
                    return []
    
        latest = {}
    
        all_packages = self.find_by_iter("match", 0)
    
        for h in all_packages:
            name = h['name']
            epoch = h['epoch']
            if epoch == None:
                epoch = ""
            version = h['version']
            release = h['release']

            # if we've never seen the package, or if the one we see is
            # NEWER than the one we've seen before, keep track of it.
            # the net result is a latest holding only the latest vre
            # of each package

            if not latest.has_key(name):
                latest[name] = { 'name' : name,
                                 'version' : version,
                                 'release' : release,
                                 'epoch': epoch }
            else:
                if self.compare_vre(latest[name]['version'],
                                    latest[name]['release'],
                                    latest[name]['epoch'],
                                    version,
                                    release,
                                    epoch) < 0:
                    latest[name] = { 'name' : name,
                                     'version' : version,
                                     'release' : release,
                                     'epoch': epoch }

        rhn_utils.log_debug("Ended RPM query for installed packages")

        self.latest_installed_package_cache = latest
        self.last_rpmdb_mtime = self.rpmdb_mtime()

        self.change_number = self.change_number + 1

        return latest
    
        
    def latest_installed_packages(self):
        return self.latest_installed_package_cache
    
    def compare_vre(self, v1, r1, e1, v2, r2, e2):
        cmp = rpm.labelCompare((str(e1), str(v1), str(r1)),
                               (str(e2), str(v2), str(r2)))
        return cmp
